import java.awt.Color;
import java.awt.Point;
import java.awt.event.KeyAdapter;
import java.awt.event.KeyEvent;
import java.util.Random;
import javax.swing.JFrame;
import javax.swing.SwingUtilities;

// Driver class
// get command line arguments and launch window
// by instantiating DrawingCanvas in the createAndShowGUI()
// method
public class ExperimentPartII {
  
  public static final int FRAME_WIDTH = 600;
  public static final int FRAME_HEIGHT = 600;
  
  private static final int RGBMAX = 256;

  private static final int NUM_OF_SHAPES = 12;
  public static int numOfShapes = NUM_OF_SHAPES;
  
  private static Random random = new Random();
  
  public static Color generateRandomColor(Color mix) {
   
    int red = random.nextInt(RGBMAX);
    int green = random.nextInt(RGBMAX);
    int blue = random.nextInt(RGBMAX);
    
    // mix the color
    if (mix != null) {
      red = (red + mix.getRed()) / 2;
      green = (green + mix.getGreen()) / 2;
      blue = (blue + mix.getBlue()) / 2;
    }
    
    Color color = new Color(red, green, blue);
    return color;
  }
  
  public static Point generateRandomLocationInWindow() {
    return new Point(
        random.nextInt(FRAME_WIDTH-RecShape.DEFAULT_SIZE),
        random.nextInt(FRAME_HEIGHT-RecShape.DEFAULT_SIZE));
   }
  
  private static void createAndShowGUI() {
    
    JFrame f = new JFrame("Experiment");
    f.setDefaultCloseOperation(JFrame.EXIT_ON_CLOSE);
    f.setSize(FRAME_WIDTH, FRAME_HEIGHT);
    
    DrawingCanvas canvas = new DrawingCanvas(numOfShapes);
    f.add(canvas);
    
    f.setBackground(Color.WHITE);
    f.setVisible(true);
    canvas.requestFocus();
  }
  
  
  public static void main (String[] args) {
    if  (args.length > 0) { 
      numOfShapes = Integer.parseInt(args[0]);
    } 
    // ADD code for command line arguments
    // first string is the total number of shapes to include
    // second the percent of rectangles, the third the percent of circles:
    //    added it should add to 100 (or not exceed if you are doing the bonus)
    // (for the bonus: the number of triangles is determined by the left over)
    System.out.println("Summary");
    System.out.println("This program draws two types of shapes");
    System.out.println("Each can be selected and dragged");
    
        System.out.println("When a shape is selected: the key + "
                         + "increases its size and the key = decreases it");
    System.out.println("In arraylist implementation a selected shape is drawn last; "
                         + "when it is released it is also drawn on top of other shapes");
    System.out.println("---------------------------------------------");
    
    SwingUtilities.invokeLater(new Runnable() {
      public void run() {
        createAndShowGUI(); 
      }
    });
    
  }
}
