import java.awt.Graphics;
import java.awt.Graphics2D;
import java.awt.RenderingHints;
import java.awt.event.KeyAdapter;
import java.awt.event.KeyEvent;
import java.awt.event.MouseEvent;
import java.awt.event.MouseListener;
import java.awt.event.MouseMotionListener;
import java.util.ArrayList;
import java.util.Random;

import javax.swing.JComponent;

public class DrawingCanvas extends JComponent {
  
  private static final int NONE = -1;
  
  private static Random random = new Random();
  
  //Provided: hard-coded two shapes
  //you will want to disable those
  private RecShape oneSquare;
  private RecShape oneCircle;
  
  //For Part I
  private RecShape[] arr;   
  //alternatively could be 
  //private ShapeInterface[] arr;
  
  //For Part II
  //private ArrayList<RecShape> displayList = new ArrayList<RecShape>();
  
  private DrawingCanvasMouseListener mouseListener;
  
  //important to keep track of selected shape in collection
  private int selectedIndex = NONE;
  
  public DrawingCanvas(int numOfShapes) {
    super();     // call the constructor of JComponent
    
    initHardCoded();    //comment out later
    
    System.out.println("canvas created " + numOfShapes);  
    arr = new RecShape[numOfShapes];
    initArray();
    
    initMouseListener();
  }
  
  private void initHardCoded() {
    oneSquare =  new MySquare(Experiment.generateRandomLocationInWindow());
    oneCircle =  new MyCircle(Experiment.generateRandomLocationInWindow());;
  }
  
  private void initArray() {
    
    // create shapes 
    //  - first start with 50% each out of NUM_OF_SHAPES = 12
    //  - later you may use amount given on the command line (not required)
    // look at helper methods of the Experiment class
    for (int i=0; i<arr.length; i=i+2) {
      arr[i] = new MyCircle(Experiment.generateRandomLocationInWindow());
      arr[i+1] = new MySquare(Experiment.generateRandomLocationInWindow());
    }
  }
  
  
  private void initMouseListener() {
    requestFocus();
    addKeyListener(new KeyAdapter() {
      
      // Keyboard controls such as q for quit
      public void keyPressed(KeyEvent e) {
        System.out.println("key " + e.getKeyChar() + " " + e.getKeyCode());
        
        if (e.getKeyCode()==KeyEvent.VK_Q)
          System.exit(1);
        
        else if (e.getKeyCode()==KeyEvent.VK_EQUALS) 
          scaleSelectedShape(1);
        
        else if (e.getKeyCode()==KeyEvent.VK_MINUS) 
          scaleSelectedShape(-1);
        
        else if (e.getKeyCode()==KeyEvent.VK_T) 
          switchSelectedShape();
        
        //For Part II
//        else if (e.getKeyCode()==KeyEvent.VK_A) 
//          addNewShape();
//        else if (e.getKeyCode()==KeyEvent.VK_D) 
//          deleteSelectedShape();
      }
    });
    mouseListener = new DrawingCanvasMouseListener();
    addMouseListener(mouseListener);
    addMouseMotionListener(mouseListener);
  }
  
  
  
  public void paintComponent(Graphics g) {
    super.paintComponent(g);
    //System.out.println("repaint all the shapes");
    
    Graphics2D g2d = (Graphics2D)g;
    g2d.setRenderingHint(RenderingHints.KEY_ANTIALIASING,
                         RenderingHints.VALUE_ANTIALIAS_ON);
    
    //Part I & Part II
    //change drawing the two hard-coded shape
    //for drawing all the ones in your arr collection
    
    oneCircle.paintComponent(g2d);  
    oneSquare.paintComponent(g2d);   
    
    
  }
  
  public void scaleSelectedShape(int val) {
    //Part I & Part II
    //scale selected shape instead of hard-coded between two
    
    if (oneCircle.getSelected())
      oneCircle.scale(val);
    else if (oneSquare.getSelected())
      oneSquare.scale(val);
    
    repaint();      
    
  }
  
  public void deselect() {
    //Part I & Part II
    //need to deselect the specific selected shape of the collection
    
    if (oneCircle.getSelected())
      oneCircle.setSelected(false);
    
    if (oneSquare.getSelected())
      oneSquare.setSelected(false);
    
    selectedIndex = NONE;
  }
  
  public void switchSelectedShape() {
    if (selectedIndex != NONE) {
      if (oneSquare.getSelected()) {
        if (oneSquare instanceof MySquare) {
          oneSquare = new MyCircle(oneSquare);
          
        } else {
          oneSquare = new MySquare(oneSquare);
        }
        oneSquare.setSelected(true);
        selectedIndex = 0;
        repaint();
        return;
      } else {
        
      }
      
    }
  }
  
  /**
   * An inner class to respond to mouse events.
   * Could be adapters instead. 
   *
   */
  private class DrawingCanvasMouseListener implements MouseListener,
    MouseMotionListener {
    
    private int newX, newY;
    
    public void mousePressed(MouseEvent event) {
      
      newX = event.getX();
      newY = event.getY();
      //System.out.println(newX + " " + newY);
      
      //Part I & Part II: change the hard-coded scenario below
      //finding the particular shape that the mouse down select if any
      //you have to traverse the collection until you find one
      
      //make sure it is last drawn that is selected first
      //try to place them all on top of each other, and
      //move one by one: trick starts by end; all including
      //first shape at index 0 can be selected
      
      //hard-coded for the two shapes
      if (oneSquare.contains(newX, newY)) {
        deselect();
        oneSquare.setSelected(true);
        selectedIndex = 0;
        repaint();
        return;
      } else if 
        (oneCircle.contains(newX, newY)) {
        deselect();
        oneCircle.setSelected(true);
        selectedIndex = 0;
        repaint();
        return;
      }
      
      //if neither is selected by mouse down the previously selected
      //shape is deselected
      deselect();
      repaint();
      
    }
    
    /**
     * dragging a shape 
     * 
     * @post the selected shape, if there is one selected, is moved
     */
    public void mouseDragged(MouseEvent event) {
      
      //Part I & Part II: change to drag whichever shape is selected
      if (selectedIndex != NONE) {
        int moveX = event.getX()-newX;
        int moveY = event.getY()-newY;
        
        if (oneCircle.getSelected())
          oneCircle.moveFrame(moveX, moveY);
        
        else if (oneSquare.getSelected())
          oneSquare.moveFrame(moveX, moveY);
        
        newX = event.getX();
        newY = event.getY();
        repaint(); 
      }
      
    }
    
    
    
    /*
     * These methods are required by the interfaces. 
     * Do nothing here
     */
    
    public void mouseReleased(MouseEvent event) { }
    
    public void mouseClicked(MouseEvent event) { }
    
    public void mouseEntered(MouseEvent event) { }
    
    public void mouseExited(MouseEvent event) { }
    
    public void mouseMoved(MouseEvent event) { }
    
  }
}
