import java.awt.Color;
import java.awt.Point;
import java.awt.event.KeyAdapter;
import java.awt.event.KeyEvent;
import java.util.Random;
import javax.swing.JFrame;
import javax.swing.SwingUtilities;

// Driver class
// could get command line arguments 
// launch window by instantiating DrawingCanvas in createAndShowGUI()

public class Experiment {
  
  public static final int FRAME_WIDTH = 600;
  public static final int FRAME_HEIGHT = 600;
  
  private static final int RGBMAX = 256;
  
  private static final int NUM_OF_SHAPES = 12;
  private static int numOfShapes = NUM_OF_SHAPES;
  
  private static Random random = new Random();
  
  public static Color generateRandomColor(Color mix) {
    
    int red = random.nextInt(RGBMAX);
    int green = random.nextInt(RGBMAX);
    int blue = random.nextInt(RGBMAX);
    
    // mix the color
    if (mix != null) {
      red = (red + mix.getRed()) / 2;
      green = (green + mix.getGreen()) / 2;
      blue = (blue + mix.getBlue()) / 2;
    }
    
    Color color = new Color(red, green, blue);
    return color;
  }
  
  public static Point generateRandomLocationInWindow() {
    return new Point(
                     random.nextInt(FRAME_WIDTH-RecShape.DEFAULT_SIZE),
                     random.nextInt(FRAME_HEIGHT-RecShape.DEFAULT_SIZE));
  }
  
  private static void createAndShowGUI() {
    
    JFrame f = new JFrame("Experiment");
    f.setDefaultCloseOperation(JFrame.EXIT_ON_CLOSE);
    f.setSize(FRAME_WIDTH, FRAME_HEIGHT);
    
    DrawingCanvas canvas = new DrawingCanvas(numOfShapes);
    f.add(canvas);
    
    f.setBackground(Color.WHITE);
    f.setVisible(true);
    canvas.requestFocus();
  }
  
  
  public static void main (String[] args) {
    if  (args.length > 0) { 
      numOfShapes = Integer.parseInt(args[0]);
    } 
    // add code for command line arguments: if you want
    // it isn't required
    // For example the first string could be the total number of shapes to include,
    // the second the percent of rectangles (the percent of circles could be the 
    // remaining)
   
    System.out.println("Overview");
    System.out.println("This program draws two types of shapes");
    System.out.println("Each can be selected and dragged");
    
    System.out.println("When a shape is selected:\n the key '=' "
                         + "increases its size\n the key '-' decreases it\n" 
                         + "the key 't' transforms it");

    System.out.println("In arraylist implementation a selected shape is drawn last; "
                         + "when it is released it is also drawn on top of other shapes");
    System.out.println("---------------------------------------------");
    
    SwingUtilities.invokeLater(new Runnable() {
      public void run() {
        createAndShowGUI(); 
      }
    });
    
  }
}
